/*-
 * Copyright (c) 2007 Akihiko GOTANDA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#ifndef __SDSHC_REG_H
#define __SDSHC_REG_H

/*
 * PCI Configuration Register
 */
#define	SDSHC_PCIR_SLOTINFO		0x40

#define	SDSHC_PCIR_CAP_CAPID		0x00
#define	SDSHC_PCIR_CAP_NEXTPTR		0x01
#define	SDSHC_PCIR_CAP_PMC		0x02
#define	SDSHC_PCIR_CAP_PMCSR		0x04
#define	SDSHC_PCIR_CAP_PMCSR_BSE	0x06
#define	SDSHC_PCIR_CAP_DATA		0x07

/*
 * SD Host Standard Register
 */
#define	SDSHC_HSR_SYSADDR		0x000	/* 4 */
#define	SDSHC_HSR_BLKSIZE		0x004	/* 2 */
#define	SDSHC_HSR_BLKCOUNT		0x006	/* 2 */
#define	SDSHC_HSR_ARGUMENT		0x008	/* 4 */
#define	SDSHC_HSR_XFERMODE		0x00c	/* 2 */
#define	SDSHC_HSR_COMMAND		0x00e	/* 2 */
#define	SDSHC_HSR_RESPONSE0		0x010	/* 4 */
#define	SDSHC_HSR_RESPONSE1		0x014	/* 4 */
#define	SDSHC_HSR_RESPONSE2		0x018	/* 4 */
#define	SDSHC_HSR_RESPONSE3		0x01c	/* 4 */
#define	SDSHC_HSR_BUFDATA_PORT		0x020	/* 4 */
#define	SDSHC_HSR_PRESENT_STATE		0x024	/* 4 */
#define	SDSHC_HSR_PRESENT_STATE_CMDLINE			0x01000000
#define	SDSHC_HSR_PRESENT_STATE_DATLINE			0x00f00000
#define	SDSHC_HSR_PRESENT_STATE_DAT3			0x00800000
#define	SDSHC_HSR_PRESENT_STATE_DAT2			0x00400000
#define	SDSHC_HSR_PRESENT_STATE_DAT1			0x00200000
#define	SDSHC_HSR_PRESENT_STATE_DAT0			0x00100000
#define	SDSHC_HSR_PRESENT_STATE_WRITE_ENABLE		0x00080000
#define	SDSHC_HSR_PRESENT_STATE_CARD_PRESENT		0x00040000
#define	SDSHC_HSR_PRESENT_STATE_CARD_STATE_STABLE	0x00020000
#define	SDSHC_HSR_PRESENT_STATE_CARD_INSERTED		0x00010000
#define	SDSHC_HSR_PRESENT_STATE_BUFRD_ENABLE		0x00000800
#define	SDSHC_HSR_PRESENT_STATE_BUFWR_ENABLE		0x00000400
#define	SDSHC_HSR_PRESENT_STATE_RD_TRANS_ACTIVE		0x00000200
#define	SDSHC_HSR_PRESENT_STATE_WR_TRANS_ACTIVE		0x00000100
#define	SDSHC_HSR_PRESENT_STATE_DATLINE_ACTIVE		0x00000004
#define	SDSHC_HSR_PRESENT_STATE_CMD_BUSY		0x00000003
#define	SDSHC_HSR_PRESENT_STATE_CMD_INH_DAT		0x00000002
#define	SDSHC_HSR_PRESENT_STATE_CMD_INH_CMD		0x00000001
#define	SDSHC_HSR_HOST_CTL		0x028	/* 1 */
#define	SDSHC_HSR_HOST_CTL_HS_EN			0x04
#define	SDSHC_HSR_HOST_CTL_4BIT_EN			0x02
#define	SDSHC_HSR_HOST_CTL_LED				0x01
#define	SDSHC_HSR_POWER_CTL		0x029	/* 1 */
#define	SDSHC_HSR_BLKGAP_CTL		0x02a	/* 1 */
#define	SDSHC_HSR_WAKEUP_CTL		0x02b	/* 1 */
#define	SDSHC_HSR_CLOCK_CTL		0x02c	/* 2 */
#define	SDSHC_HSR_CLOCK_CTL_SD_CLK_EN			0x04
#define	SDSHC_HSR_CLOCK_CTL_INTERNAL_CLK_STABLE		0x02
#define	SDSHC_HSR_CLOCK_CTL_INTERNAL_CLK_EN		0x01
#define	SDSHC_HSR_TIMEOUT_CTL		0x02e	/* 1 */
#define	SDSHC_HSR_RESET			0x02f	/* 1 */
#define	SDSHC_HSR_RESET_LINE				0x06
#define	SDSHC_HSR_RESET_DAT				0x04
#define	SDSHC_HSR_RESET_CMD				0x02
#define	SDSHC_HSR_RESET_ALL				0x01
#define	SDSHC_HSR_RESET_MASK				0x07
#define	SDSHC_HSR_NORM_INTR_ST		0x030	/* 2 */
#define	SDSHC_HSR_NORM_INTR_ERR_INT			0x8000
#define	SDSHC_HSR_NORM_INTR_CARD_INT			0x0100
#define	SDSHC_HSR_NORM_INTR_CARD_DETECT			0x00c0
#define	SDSHC_HSR_NORM_INTR_CARD_REMOVE			0x0080
#define	SDSHC_HSR_NORM_INTR_CARD_INSERT			0x0040
#define	SDSHC_HSR_NORM_INTR_BUFRD_READY			0x0020
#define	SDSHC_HSR_NORM_INTR_BUFWR_READY			0x0010
#define	SDSHC_HSR_NORM_INTR_DMA_INTR			0x0008
#define	SDSHC_HSR_NORM_INTR_BLKGAP_EVENT		0x0004
#define	SDSHC_HSR_NORM_INTR_TRANS_COMPLETE		0x0002
#define	SDSHC_HSR_NORM_INTR_CMD_COMPLETE		0x0001
#define	SDSHC_HSR_ERR_INTR_ST		0x032	/* 2 */
#define	SDSHC_HSR_ERR_INTR_VENDOR			0xf000
#define	SDSHC_HSR_ERR_INTR_AUTO_CMD12			0x0010
#define	SDSHC_HSR_ERR_INTR_CURRENT_LIMIT		0x0080
#define	SDSHC_HSR_ERR_INTR_DATA_ERROR			0x0070
#define	SDSHC_HSR_ERR_INTR_DATA_END_BIT			0x0040
#define	SDSHC_HSR_ERR_INTR_DATA_CRC			0x0020
#define	SDSHC_HSR_ERR_INTR_DATA_TIMEOUT			0x0010
#define	SDSHC_HSR_ERR_INTR_CMD_ERROR			0x000f
#define	SDSHC_HSR_ERR_INTR_CMD_INDEX			0x0008
#define	SDSHC_HSR_ERR_INTR_CMD_END_BIT			0x0004
#define	SDSHC_HSR_ERR_INTR_CMD_CRC			0x0002
#define	SDSHC_HSR_ERR_INTR_CMD_TIMEOUT			0x0001
#define	SDSHC_HSR_NORM_INTR_EN		0x034	/* 2 */
#define	SDSHC_HSR_ERR_INTR_EN		0x036	/* 2 */
#define	SDSHC_HSR_NORM_INTR_SIGEN	0x038	/* 2 */
#define	SDSHC_HSR_ERR_INTR_SIGEN	0x03a	/* 2 */
#define	SDSHC_HSR_AUTOCMD12_ERR		0x03c	/* 2 */
#define	SDSHC_HSR_CAPABILITIES		0x040	/* 4 */
#define	SDSHC_HSR_CAP_RESERVED		0x044	/* 4 */
#define	SDSHC_HSR_MAXCURR_CAP		0x048	/* 4 */
#define	SDSHC_HSR_MAXCURR_CAP_RES	0x04c	/* 4 */
#define	SDSHC_HSR_SLOT_INTR_STATUS	0x0fc	/* 2 */
#define	SDSHC_HSR_HOST_CTLR_VER		0x0fe	/* 2 */


#define	SDSHC_MAX_BLOCK_COUNT	0x0ffff


#endif /* __SDSHC_REG_H */
