/*-
 * Copyright (c) 2007 Akihiko GOTANDA
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#ifndef __SDVAR_H
#define __SDVAR_H

#include <sys/param.h>
#include <sys/bus.h>
#include <sys/lock.h>
#include <sys/mutex.h>
#include <sys/rman.h>
#include <sys/taskqueue.h>

extern int sdshc_timeout_interval;
extern int sdshc_polling_interval;
extern int sdshc_wait_after_insertion;
extern int sdshc_use_highspeed_mode;

#if defined(SD_DEBUG)
extern int sdshc_debug_level;
# define LOG(lv, x)	if ((lv) <= sdshc_debug_level)	device_printf x
#else
# define LOG(lv, x)
#endif
#define MSG(x)		device_printf x


#define	SDSHC_MAXNUM_SLOT	6
#define	SDSHC_SLOT_BITNUM	3

#define	SDSHC_TIMEOUT_COUNT	100

#define SDSHC_INITIAL_CLOCK	300	/* unit: kHz */


struct sdshc_softc {
    device_t		dev;
    uint32_t		quirk_device;

    struct mtx		mtx;
    int			locked;

    /* shared area */
    int			slot_base;
    int			num_slot;
    int			irq_rid;
    struct resource	*irq;
    void 		*ih;

    struct cdev		*cdev;

    int			mem_rid[SDSHC_MAXNUM_SLOT];
    struct resource	*mem;
    bus_space_tag_t	bt;
    bus_space_handle_t	bh;

    int			flag;
#define	SDSHC_STAT_OPENED		0x01
};


struct sdshcbus_flags {
    int		in_error_recovery;
    int		error_recovery_wo_abort;
    int		use_polling;
};

struct sdshcbus_softc {
    device_t		dev;

    struct mtx		mtx;
    int			locked;

    int			slot_no[2];

    struct resource	*mem;
    bus_space_tag_t	bt;
    bus_space_handle_t	bh;

    int			support_voltage;
    int			suspend_capable;
    int			dma_capable;
    int			highspeed_capable;
    int			highspeed_enabled;
    int			base_freq;
    int			timeout_clock_freq;
    int			max_block_size;
    int			block_size;
    int			max_block_count;
    int			card_inserted;
#define	SDDEV_INVALID	-1
#define	SDDEV_NONE	 0
#define	SDDEV_UNKNOWN	 1
#define	SDDEV_MMC	0x0110
#define	SDDEV_MMCHD	0x0120
#define	SDDEV_SD1	0x0210
#define	SDDEV_SD2	0x0220
#define	SDDEV_SDHC	0x0230
#define	SDDEV_SDIO	0x1000
#define	is_card_mmc(id)		(((id) & 0x0f00) == 0x0100)
#define	is_card_sdmem(id)	(((id) & 0x0f00) == 0x0200)
#define	is_card_sdio(id)	(((id) & 0xf000) == 0x1000)
    int			card_voltage;
    int			card_max_clock;
    int			card_max_buswidth;
    int			card_clock;
    int			card_buswidth;
    u_int32_t		card_capacity;	/* unit: 512B */
    u_int32_t		card_read_access_time;	/* unit: ns */
    int			card_write_speed_factor;
    int			card_speed_class;

    int			mid;
    char		oid[3];
    char		pnm[7];
    int			prv[2];

    int			ocr;
    int			rca;
    int			sd_spec;
    int			status;
#define	SDSHCBUS_STAT_NONE			0x00
#define SDSHCBUS_STAT_COMMAND_COMPLETE		0x01
#define SDSHCBUS_STAT_TRANSFER_COMPLETE		0x02
#define	SDSHCBUS_STAT_OPENED			0x10
    u_int32_t		result;
    u_int32_t		error;

    u_int8_t		*addr;
    int			rest_blk_count;

    struct task		attach_task;
    struct task		detach_task;

    struct cdev		*cdev;
    struct sdshcbus_flags	flag;
};


#define SD_LOCK(sc)		((sc)->locked = 1, mtx_lock(&(sc)->mtx))
#define SD_UNLOCK(sc)		((sc)->locked = 0, mtx_unlock(&(sc)->mtx))


#if defined(SD_USE_BUSYLOOP)
#  define SD_SLEEP(sc, type, desc, timeout)
#else
#  define SD_SLEEP(sc, type, desc, timeout)	\
		((sc)->locked \
			? msleep((sc), &(sc)->mtx, (type), (desc), (timeout)) \
			: tsleep((sc), (type), (desc), (timeout)))
#endif

#endif /* __SDVAR_H */
